#m: matrix containing the values that are plotted, for each column a group of bars is plotted
#beside: should the group of bars be plotted beside each other or stacked?
#log: log y-axis?
#col: color of the bars (see col4subBars)
#xlab: label of the x-axis
#add: should the plot be added to an existing plot?
#ylab: label of the y-axis
#xlim: limits of the x-axis
#ylim: limits of the y-axis
#density: density for shaded bars
#barwidth: width of bars
#space: space between the bar-groups
#names: names of the bar-groups
#noAxes: draw no axes
#col4subBars: should the colors be used to color bars in a group or different bar-groups? tries to determine what to do by comparing the length of col to the dimensions of m, if not possible uses bars in a group

barplot<-function(m,beside=T,log=F,col=NULL,xlab="",add=F,ylab="",xlim=NA,ylim=NA,density=NULL, barwidth=NA, space=0.2, names=NULL, noAxes=F, col4subBars=NA, ...){
	min<-0
	max<-max(m)
	if(is.vector(m)){
		nbars<-length(m)
		nsubbars<-1
		if(is.null(names)){
			names<-names(m)
		}
		beside=T
	}
	else{
		m<-as.matrix(m)
		nbars<-ncol(m)
		nsubbars<-nrow(m)
		if(!beside){
			nsubbars<-1
			max<-max(colSums(m))
		}
		if(is.null(names)){
			names<-colnames(m)
		}
	}
	if(is.na(col4subBars)){
		if(is.vector(m) || dim(m)[1]==1){
			col4subBars=F
		}
		else if((dim(m)[1]%%length(col))==0 && (dim(m)[2]%%length(col))!=0){
			col4subBars=T
		}
		else if((dim(m)[1]%%length(col))!=0 && (dim(m)[2]%%length(col))==0){
			col4subBars=F
		}
		else{
			col4subBars=T
		}
	}
	logsym<-""
	if(log | (add==T & par("ylog"))){
		logsym<-"y"
		if(sum(m<=0,na.rm=T)>0){
			warning("removed values <=0 from data because of log")
			m[m<=0]<-NA
		}
		if(min(m,na.rm=T)>=1){
			min<-1
		}
		else{
			min<-min(m,na.rm=T)*0.5
		}
	}
	if(all(is.na(barwidth))){
		barsep<-space
		barwidth=rep((1-barsep)/nsubbars,nsubbars)
	}
	else{
		if(beside){
			barsep<-space*sum(barwidth)
		}
		else{
			barsep<-space*mean(barwidth)
		}
	}
	if(any(is.na(xlim))){
		xlim<-c(0.5,(sum(barwidth)+barsep)*nbars+0.5)
	}
	if(any(is.na(ylim))){
		ylim<-c(min,max)
	}
	if(add){
		
	}
	if(!add){
		plot(1,type="n",xlim=xlim,ylim=ylim,log=logsym,axes=F,xlab=xlab,ylab=ylab,...)
	}
	if(!noAxes){
		axis(2,xpd=T)
		axis(1,labels=names,at=seq(xlim[1]+barsep/2+sum(barwidth)/2,xlim[2],sum(barwidth)+barsep),xpd=T,...)
		box()
	}
	xstart<-xlim[1]+barsep/2
	for(i in 1:nbars){
		if(beside){
			for(j in 1:nsubbars){
				if(is.vector(m)){
					val<-m[i]
				}
				else{
					val<-m[,i]
				}
				if(col4subBars){
					colindex=j
				}
				else{
					colindex=i
				}
				rect(xstart,min,xstart+barwidth[j],val[j],col=col[colindex],density=density)
#				cat(paste("rect(",xstart,",",min,",",xstart+barwidth,",",val,")","\n",sep=""))
				xstart<-xstart+barwidth[j]
			}
		}
		else{
			y1<-c(min,cumsum(m[,i])[-nrow(m)])
			rect(xstart,y1,xstart+barwidth,cumsum(m[,i]),col=col)
			xstart<-xstart+sum(barwidth)
		}
		xstart<-xstart+barsep
	}
}
