wordlayout<-function (x, y, words, cex = 1, rotate90 = FALSE, xlim = c(-Inf, Inf), ylim = c(-Inf, Inf), tstep = 0.1, rstep = 0.1, ...) 
{
    tails <- "g|j|p|q|y"
    n <- length(words)
    sdx <- sd(x, na.rm = TRUE)
    sdy <- sd(y, na.rm = TRUE)
    if (sdx == 0) 
        sdx <- 1
    if (sdy == 0) 
        sdy <- 1
    if (length(cex) == 1) 
        cex <- rep(cex, n)
    if (length(rotate90) == 1) 
        rotate90 <- rep(rotate90, n)
    boxes <- list()
    for (i in 1:length(words)) {
cat("now placing label for ",words[i],"\n")
        rotWord <- rotate90[i]
        r <- 0
        theta <- runif(1, 0, 2 * pi)
 x1 <- xo <- x[i]
        y1 <- yo <- y[i]
        wid <- strwidth(words[i], cex = cex[i], ...)
        ht <- strheight(words[i], cex = cex[i], ...)
        if (grepl(tails, words[i])) 
            ht <- ht + ht * 0.2
        if (rotWord) {
            tmp <- ht
            ht <- wid
            wid <- tmp
        }
        isOverlaped <- TRUE
        while (isOverlaped) {
            if (!overlap(x1 - 0.5 * wid, y1 - 0.5 * ht, wid, 
                ht, boxes) && x1 - 0.5 * wid > xlim[1] && y1 - 
                0.5 * ht > ylim[1] && x1 + 0.5 * wid < xlim[2] && 
                y1 + 0.5 * ht < ylim[2]) {
	cat(overlap(x1 - 0.5 * wid, y1 - 0.5 * ht, wid,ht,boxes),"\n")
                boxes[[length(boxes) + 1]] <- c(x1 - 0.5 * wid, 
                  y1 - 0.5 * ht, wid, ht)
cat("label is placed at (",(x1-0.5*wid),",",(x1+0.5*wid),",",(y1-0.5*ht),",",(y1+0.5*ht),"\n")
                isOverlaped <- FALSE
            }
            else {
                theta <- theta + tstep
               r <- r + rstep * tstep/(2 * pi)
                x1 <- xo + sdx * r * cos(theta)
                y1 <- yo + sdy * r * sin(theta)
            }
        }
    }
    result <- do.call(rbind, boxes)
    colnames(result) <- c("x", "y", "width", "ht")
    rownames(result) <- words
    result
}

overlap<-function (x11, y11, sw11, sh11, boxes1) 
{
	overlap<-F
	left<-x11
	right<-x11+sw11
	up<-y11+sh11
	bottom<-y11
	for(box in boxes1){
		boxleft<-box[1]
		boxright<-box[1]+box[3]
		boxup<-box[2]+box[4]
		boxbottom<-box[2]
		overlapsX<-(boxleft>=left && boxleft<=right) || (boxright>=left && boxright<=right)
		overlapsY<-(boxbottom>=bottom && boxbottom<=up) || (boxup>=bottom && boxup<=up)
		if(overlapsX && overlapsY){
			overlap<-T
			cat("overlaps: (",left,",",right,",",up,",",bottom,") and box (",boxleft,",",boxright,",",boxup,",",boxbottom,")\n")
			break
		}
	}
	overlap
}

addLabels<-function(x,y,labelX, labelY, labels,...){
	layout<-wordlayout(c(x,labelX),c(y,labelY),c(rep("o",length(x)),labels))
	layoutX<-layout[,1]+0.5*layout[,3]
	layoutY<-layout[,2]+0.5*layout[,4]
	ind<-(length(x)+1):nrow(layout)
	text(layoutX[ind],layoutY[ind],labels,...)
}
