trimString<-function(x){
	gsub(gsub(x,pattern="^ +",replacement="",perl=T),pattern=" +$", replacement="",perl=T)
}

getConfig<-function(configPath){
	config<-read.table(configPath, sep=":", comment.char="#", stringsAsFactors=F)
	params<-trimString(config[,2])
	names(params)<-trimString(config[,1])
	params
}

#params: named vector that contains the params that are given
#paramNames: vector of names of the parameters that should be checked
#defaults: named vector that contains default parameter values
#stopIfNotExists: boolean indicating whether the function should stop and return an error if a parameter is not set
checkIfParamsExist<-function(params, paramNames, defaults=NA, numericParams=NULL, logicalParams=NULL, stopIfNotExists=T){
	params<-checkForType(params,numericParams,typefunction=as.numeric)
	params<-checkForType(params,logicalParams,typefunction=as.logical)
	notSet<-which(is.na(params[paramNames]))
	if(length(notSet)==0)
	{
		return(params);
	}
	notSetNames<-paramNames[notSet]
	unsetNames<-NULL
	for(nsName in notSetNames)
	{
		if(!is.na(defaults) && !is.na(defaults[nsName]))
		{
			params[nsName]<-defaults[nsName]
		}
		else
		{
			unsetNames<-c(unsetNames,nsName)
		}
	}
	if(length(unsetNames)>0 && stopIfNotExists)
	{
		stop(paste(unsetNames,collapse=" and ")," is/are not set\n")
	}
	return(params)
}

checkForType<-function(params, paramNames, typefunction=as.numeric)
{
	notNum<-which(is.na(typefunction(params[paramNames])))
	if(length(notNum)==0)
	{
		return(params);
	}
	indexRightType<-which(!is.element(names(params),paramNames[notNum]))
	params<-params[indexRightType]
	return(params);
}

startDevice<-function(type, outpath)
{
	library(GDD)
	GDD(file=outpath,type=type)
#	if(type=="png")
#	{
#		png(outpath)
#	}else if(type=="ps")
#	{
#		ps(outpath)
#	}else if(type=="pdf")
#	{
#		pdf(outpath)
#	}
}

getAxesCoordinates<-function()
{
	axes<-par("usr")
	my.x<-axes[1:2]
	my.y<-axes[3:4]
	if(par("xlog"))
	{
		my.x<-10^my.x
	}
	if(par("ylog"))
	{
		my.y<-10^my.y
	}
	dev.x<-grconvertX(my.x,from="user",to="device")
	dev.y<-grconvertY(my.y,from="user",to="device")
	list("my.x"=my.x,"my.y"=my.y,"dev.x"=dev.x,"dev.y"=dev.y)
}

